"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.assertValidityOfFeeCurve = exports.balancingFeeFunctionLookupMapping = exports.computePiecewiseLinearFunction = exports.getDepositBalancingFee = exports.getRefundBalancingFee = exports.getInterval = exports.getBounds = exports.performLinearIntegration = void 0;
var tslib_1 = require("tslib");
var utils_1 = require("../utils");
var constants_1 = require("../constants");
function performLinearIntegration(cutoffArray, index, integralStart, integralEnd) {
    var lengthUnderCurve = integralEnd.sub(integralStart);
    var resolveValue = function (index) { return cutoffArray[index][1]; };
    var feeIntegral = resolveValue(Math.max(0, Math.min(cutoffArray.length - 1, index - 1)))
        .mul(lengthUnderCurve)
        .div(utils_1.fixedPointAdjustment);
    if (index > 0 && index < cutoffArray.length - 1) {
        var _a = cutoffArray[index], currCutoff = _a[0], currValue = _a[1];
        var _b = cutoffArray[index - 1], prevCutoff = _b[0], prevValue = _b[1];
        var slope = prevValue.sub(currValue).mul(utils_1.fixedPointAdjustment).div(prevCutoff.sub(currCutoff));
        var integralEndExpression = integralEnd.pow(2).div(2).sub(prevCutoff.mul(integralEnd));
        var integralStartExpression = integralStart.pow(2).div(2).sub(prevCutoff.mul(integralStart));
        var slopeIntegration = slope
            .mul(integralEndExpression.sub(integralStartExpression))
            .div(utils_1.fixedPointAdjustment)
            .div(utils_1.fixedPointAdjustment);
        feeIntegral = feeIntegral.add(slopeIntegration);
    }
    return feeIntegral;
}
exports.performLinearIntegration = performLinearIntegration;
function getBounds(cutoffArray, index) {
    var length = cutoffArray.length;
    if (index === 0) {
        return [constants_1.UBA_BOUNDS_RANGE_MIN, cutoffArray[0][0]];
    }
    else if (index < length) {
        return [cutoffArray[index - 1][0], cutoffArray[index][0]];
    }
    else {
        return [cutoffArray[length - 1][0], constants_1.UBA_BOUNDS_RANGE_MAX];
    }
}
exports.getBounds = getBounds;
function getInterval(cutoffArray, target) {
    var result = [-1, [constants_1.UBA_BOUNDS_RANGE_MIN, constants_1.UBA_BOUNDS_RANGE_MAX]];
    for (var i = 0; i <= cutoffArray.length; i++) {
        var _a = getBounds(cutoffArray, i), lowerBound = _a[0], upperBound = _a[1];
        if (target.gte(lowerBound) && target.lt(upperBound)) {
            result = [i, [lowerBound, upperBound]];
            break;
        }
    }
    return result;
}
exports.getInterval = getInterval;
function getRefundBalancingFee(cutoffArray, runningBalance, modificationAmount) {
    var _a = getInterval(cutoffArray, runningBalance), balanceIndex = _a[0], balanceLowerBound = _a[1][0];
    var _b = getInterval(cutoffArray, runningBalance.sub(modificationAmount)), balanceLessModificationIndex = _b[0], _c = _b[1], balanceLessModificationUpperBound = _c[1];
    var totalFee = (0, utils_1.toBN)(0);
    for (var index = balanceIndex; index >= balanceLessModificationIndex; index--) {
        var integralStart = void 0;
        var integralEnd = void 0;
        if (index === balanceIndex && index === balanceLessModificationIndex) {
            integralStart = runningBalance;
            integralEnd = runningBalance.sub(modificationAmount);
        }
        else if (index === balanceIndex) {
            integralStart = runningBalance;
            integralEnd = balanceLowerBound;
        }
        else if (index === balanceLessModificationIndex) {
            integralStart = balanceLessModificationUpperBound;
            integralEnd = runningBalance.sub(modificationAmount);
        }
        else {
            var _d = getBounds(cutoffArray, index), lowerBound = _d[0], upperBound = _d[1];
            integralStart = lowerBound;
            integralEnd = upperBound;
        }
        totalFee = totalFee.add(performLinearIntegration(cutoffArray, index, integralStart, integralEnd));
    }
    return totalFee;
}
exports.getRefundBalancingFee = getRefundBalancingFee;
function getDepositBalancingFee(cutoffArray, runningBalance, modificationAmount) {
    var _a = getInterval(cutoffArray, runningBalance), balanceIndex = _a[0], _b = _a[1], balanceUpperBound = _b[1];
    var _c = getInterval(cutoffArray, runningBalance.add(modificationAmount)), balancePlusModificationIndex = _c[0], balancePlusModificationLowerBound = _c[1][0];
    var totalFee = (0, utils_1.toBN)(0);
    for (var index = balanceIndex; index <= balancePlusModificationIndex; index++) {
        var integralStart = void 0;
        var integralEnd = void 0;
        if (index === balanceIndex && index === balancePlusModificationIndex) {
            integralStart = runningBalance;
            integralEnd = runningBalance.add(modificationAmount);
        }
        else if (index === balanceIndex) {
            integralStart = runningBalance;
            integralEnd = balanceUpperBound;
        }
        else if (index === balancePlusModificationIndex) {
            integralStart = balancePlusModificationLowerBound;
            integralEnd = runningBalance.add(modificationAmount);
        }
        else {
            var _d = getBounds(cutoffArray, index), lowerBound = _d[0], upperBound = _d[1];
            integralStart = lowerBound;
            integralEnd = upperBound;
        }
        totalFee = totalFee.add(performLinearIntegration(cutoffArray, index, integralStart, integralEnd));
    }
    return totalFee;
}
exports.getDepositBalancingFee = getDepositBalancingFee;
function minBN(a, b) {
    return a.lt(b) ? a : b;
}
function maxBN(a, b) {
    return a.gt(b) ? a : b;
}
function minMaxBN(a, b) {
    return [minBN(a, b), maxBN(a, b)];
}
function computePiecewiseLinearFunction(functionBounds, x, y) {
    functionBounds = tslib_1.__spreadArray(tslib_1.__spreadArray([], functionBounds, true), [[constants_1.UBA_BOUNDS_RANGE_MAX, functionBounds[functionBounds.length - 1][1]]], false);
    var xBar = functionBounds.map(function (_, idx, arr) { return getBounds(arr, idx); });
    var scale = x.lte(y) ? 1 : -1;
    var _a = minMaxBN(x, y), lb = _a[0], ub = _a[1];
    var _b = [getInterval(functionBounds, lb)[0], getInterval(functionBounds, ub)[0]], lbIdx = _b[0], ubIdx = _b[1];
    var integral = (0, utils_1.toBN)(0);
    for (var idx = lbIdx; idx <= ubIdx; idx++) {
        var _lb = idx == lbIdx ? lb : xBar[idx][0];
        var _ub = idx == ubIdx ? ub : xBar[idx][1];
        if (!_lb.eq(_ub)) {
            var currentIntegration = performLinearIntegration(functionBounds, idx, _lb, _ub);
            integral = integral.add(currentIntegration);
        }
    }
    return integral.mul(scale);
}
exports.computePiecewiseLinearFunction = computePiecewiseLinearFunction;
exports.balancingFeeFunctionLookupMapping = {
    inflow: getDepositBalancingFee,
    outflow: getRefundBalancingFee,
};
function assertValidityOfFeeCurve(feeCurve, validateZeroPoint) {
    if (feeCurve.length === 0) {
        throw new Error("Balancing fee curve must have at least one point");
    }
    if (feeCurve.some(function (tuple) { return tuple.length !== 2; })) {
        throw new Error("Balancing fee curve must be a list of tuples");
    }
    if (validateZeroPoint && !feeCurve.some(function (tuple) { return tuple[1].eq(0); })) {
        throw new Error("Balancing fee curve must have a zero point");
    }
    if (feeCurve.some(function (tuple, idx, arr) { return idx > 0 && tuple[0].lte(arr[idx - 1][0]); })) {
        throw new Error("Balancing fee curve must have strictly increasing x values");
    }
    if (feeCurve.some(function (tuple, idx, arr) { return idx > 0 && tuple[1].lt(arr[idx - 1][1]); })) {
        throw new Error("Balancing fee curve must have increasing y values");
    }
}
exports.assertValidityOfFeeCurve = assertValidityOfFeeCurve;
//# sourceMappingURL=UBAFeeUtility.js.map