"use strict";
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.TransfersRepository = void 0;
var tslib_1 = require("tslib");
var model_1 = require("../web3/model");
var usersTransfersStoreInitialValue = {};
var transfersStoreInitialValue = (_a = {},
    _a[model_1.CHAIN_IDs.GOERLI] = {},
    _a[model_1.CHAIN_IDs.POLYGON] = {},
    _a[model_1.CHAIN_IDs.MAINNET] = {},
    _a);
var TransfersRepository = (function () {
    function TransfersRepository() {
        this.transfers = transfersStoreInitialValue;
        this.usersTransfers = usersTransfersStoreInitialValue;
    }
    TransfersRepository.prototype.insertTransfer = function (chainId, depositorAddr, depositId, transfer) {
        if (!this.transfers[chainId]) {
            this.transfers[chainId] = {};
        }
        if (!this.transfers[chainId][depositorAddr]) {
            this.transfers[chainId][depositorAddr] = {};
        }
        if (this.transfers[chainId][depositorAddr][depositId]) {
            return;
        }
        this.transfers[chainId][depositorAddr][depositId] = transfer;
    };
    TransfersRepository.prototype.updateFilledAmount = function (chainId, depositorAddr, depositId, filled, fillTxHash, appliedRelayerFeePct) {
        var _a, _b, _c;
        var transfer = (_c = (_b = (_a = this.transfers) === null || _a === void 0 ? void 0 : _a[chainId]) === null || _b === void 0 ? void 0 : _b[depositorAddr]) === null || _c === void 0 ? void 0 : _c[depositId];
        if (!transfer) {
            console.error("couldn't fill deposit on chain ".concat(chainId, ", depositId ").concat(depositId, ", depositor ").concat(depositorAddr));
            return;
        }
        this.transfers[chainId][depositorAddr][depositId] = tslib_1.__assign(tslib_1.__assign({}, transfer), { filled: filled, status: transfer.amount.eq(filled) ? "filled" : "pending", fillTxs: Array.from(new Set(tslib_1.__spreadArray(tslib_1.__spreadArray([], transfer.fillTxs, true), [fillTxHash], false))), currentRelayerFeePct: appliedRelayerFeePct });
    };
    TransfersRepository.prototype.updateRelayerFee = function (sourceChainId, depositorAddr, depositId, newRelayerFeePct, speedUpTxHash, timestamp) {
        var _a, _b, _c;
        var transfer = (_c = (_b = (_a = this.transfers) === null || _a === void 0 ? void 0 : _a[sourceChainId]) === null || _b === void 0 ? void 0 : _b[depositorAddr]) === null || _c === void 0 ? void 0 : _c[depositId];
        if (!transfer) {
            console.error("couldn't update 'relayerFee' on chain ".concat(sourceChainId, ", depositId ").concat(depositId, ", depositor ").concat(depositorAddr));
            return;
        }
        this.transfers[transfer.sourceChainId][depositorAddr][depositId] = tslib_1.__assign(tslib_1.__assign({}, transfer), { currentRelayerFeePct: transfer.status === "pending" ? newRelayerFeePct : transfer.currentRelayerFeePct, speedUps: tslib_1.__spreadArray(tslib_1.__spreadArray([], transfer.speedUps.filter(function (speedUp) { return speedUp.txHash !== speedUpTxHash; }), true), [
                {
                    txHash: speedUpTxHash,
                    relayerFeePct: newRelayerFeePct,
                    timestamp: timestamp,
                },
            ], false) });
    };
    TransfersRepository.prototype.getTransfersByChainAndDepositor = function (chainId, depositorAddr) {
        if (this.transfers[chainId][depositorAddr]) {
            return Object.values(this.transfers[chainId][depositorAddr]);
        }
        return [];
    };
    TransfersRepository.prototype.setDepositorPendingTransfers = function (depositorAddr, transfers) {
        if (!this.usersTransfers[depositorAddr]) {
            this.usersTransfers[depositorAddr] = { pending: [], filled: [] };
        }
        this.usersTransfers[depositorAddr].pending = transfers;
    };
    TransfersRepository.prototype.setDepositorFilledTransfers = function (depositorAddr, transfers) {
        if (!this.usersTransfers[depositorAddr]) {
            this.usersTransfers[depositorAddr] = { filled: [], pending: [] };
        }
        this.usersTransfers[depositorAddr].filled = transfers;
    };
    TransfersRepository.prototype.aggregateTransfers = function () {
        var _this = this;
        var chainsIds = Object.keys(this.transfers).map(function (chainId) { return parseInt(chainId); });
        var depositors = chainsIds.reduce(function (acc, chainId) { return tslib_1.__spreadArray(tslib_1.__spreadArray([], acc, true), Object.keys(_this.transfers[chainId]), true); }, []);
        for (var _i = 0, depositors_1 = depositors; _i < depositors_1.length; _i++) {
            var depositorAddr = depositors_1[_i];
            var transfers = [];
            for (var _a = 0, chainsIds_1 = chainsIds; _a < chainsIds_1.length; _a++) {
                var chainId = chainsIds_1[_a];
                transfers.push.apply(transfers, this.getTransfersByChainAndDepositor(chainId, depositorAddr));
            }
            transfers.sort(function (t1, t2) { return t2.depositTime - t1.depositTime; });
            var filteredTransfers = transfers.reduce(function (acc, transfer) { return ({
                pending: transfer.status === "pending" ? tslib_1.__spreadArray(tslib_1.__spreadArray([], acc.pending, true), [transfer], false) : acc.pending,
                filled: transfer.status === "filled" ? tslib_1.__spreadArray(tslib_1.__spreadArray([], acc.filled, true), [transfer], false) : acc.filled,
            }); }, { pending: [], filled: [] });
            this.setDepositorFilledTransfers(depositorAddr, filteredTransfers.filled);
            this.setDepositorPendingTransfers(depositorAddr, filteredTransfers.pending);
        }
    };
    TransfersRepository.prototype.getFilledTransfers = function (depositorAddr, limit, offset) {
        var _a;
        var transfers = ((_a = this.usersTransfers[depositorAddr]) === null || _a === void 0 ? void 0 : _a.filled) || [];
        if (transfers.length === 0) {
            return transfers;
        }
        return transfers.slice(offset, limit && offset ? limit + offset : limit);
    };
    TransfersRepository.prototype.getAllFilledTransfers = function (limit, offset) {
        return Object.values(this.usersTransfers)
            .flatMap(function (t) { return t.filled; })
            .sort(function (t1, t2) { return t2.depositTime - t1.depositTime; })
            .slice(offset, limit && offset ? limit + offset : limit);
    };
    TransfersRepository.prototype.getPendingTransfers = function (depositorAddr, limit, offset) {
        var _a;
        var transfers = ((_a = this.usersTransfers[depositorAddr]) === null || _a === void 0 ? void 0 : _a.pending) || [];
        if (transfers.length === 0) {
            return transfers;
        }
        return transfers.slice(offset, limit && offset ? limit + offset : limit);
    };
    TransfersRepository.prototype.getAllPendingTransfers = function (limit, offset) {
        return Object.values(this.usersTransfers)
            .flatMap(function (t) { return t.pending; })
            .sort(function (t1, t2) { return t2.depositTime - t1.depositTime; })
            .slice(offset, limit && offset ? limit + offset : limit);
    };
    TransfersRepository.prototype.countFilledTransfers = function (depositorAddr) {
        var _a;
        var transfers = ((_a = this.usersTransfers[depositorAddr]) === null || _a === void 0 ? void 0 : _a.filled) || [];
        return transfers.length;
    };
    TransfersRepository.prototype.countAllFilledTransfers = function () {
        return Object.values(this.usersTransfers).flatMap(function (t) { return t.filled; }).length;
    };
    TransfersRepository.prototype.countPendingTransfers = function (depositorAddr) {
        var _a;
        var transfers = ((_a = this.usersTransfers[depositorAddr]) === null || _a === void 0 ? void 0 : _a.pending) || [];
        return transfers.length;
    };
    TransfersRepository.prototype.countAllPendingTransfers = function () {
        return Object.values(this.usersTransfers).flatMap(function (t) { return t.pending; }).length;
    };
    return TransfersRepository;
}());
exports.TransfersRepository = TransfersRepository;
//# sourceMappingURL=transfers-repository.js.map