"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.queryHistoricalDepositForFill = void 0;
var tslib_1 = require("tslib");
var assert_1 = tslib_1.__importDefault(require("assert"));
var FlowUtils_1 = require("./FlowUtils");
var TimeUtils_1 = require("./TimeUtils");
var TypeGuards_1 = require("./TypeGuards");
var CachingUtils_1 = require("./CachingUtils");
var constants_1 = require("../constants");
function queryHistoricalDepositForFill(spokePoolClient, fill, cache) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var deposit, cachedDeposit;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (fill.originChainId !== spokePoolClient.chainId) {
                        throw new Error("OriginChainId mismatch (".concat(fill.originChainId, " != ").concat(spokePoolClient.chainId, ")"));
                    }
                    if (!spokePoolClient.isUpdated) {
                        throw new Error("SpokePoolClient must be updated before querying historical deposits");
                    }
                    if (fill.depositId < spokePoolClient.firstDepositIdForSpokePool ||
                        fill.depositId > spokePoolClient.lastDepositIdForSpokePool) {
                        return [2, undefined];
                    }
                    if (fill.depositId >= spokePoolClient.earliestDepositIdQueried &&
                        fill.depositId <= spokePoolClient.latestDepositIdQueried) {
                        return [2, spokePoolClient.getDepositForFill(fill)];
                    }
                    if (!cache) return [3, 2];
                    return [4, (0, CachingUtils_1.getDepositInCache)((0, CachingUtils_1.getDepositKey)(fill), cache)];
                case 1:
                    cachedDeposit = _a.sent();
                    _a.label = 2;
                case 2:
                    if (!(0, TypeGuards_1.isDefined)(cachedDeposit)) return [3, 3];
                    deposit = cachedDeposit;
                    (0, assert_1.default)(deposit.depositId === fill.depositId && deposit.originChainId === fill.originChainId);
                    return [3, 6];
                case 3: return [4, spokePoolClient.findDeposit(fill.depositId, fill.destinationChainId, fill.depositor)];
                case 4:
                    deposit = _a.sent();
                    if (!cache) return [3, 6];
                    return [4, (0, CachingUtils_1.setDepositInCache)(deposit, (0, TimeUtils_1.getCurrentTime)(), cache, constants_1.DEFAULT_CACHING_TTL)];
                case 5:
                    _a.sent();
                    _a.label = 6;
                case 6: return [2, (0, FlowUtils_1.validateFillForDeposit)(fill, deposit) ? deposit : undefined];
            }
        });
    });
}
exports.queryHistoricalDepositForFill = queryHistoricalDepositForFill;
//# sourceMappingURL=DepositUtils.js.map